@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2005 Rocky Bernstein <rocky@panix.com>
+ * Copyright (C) 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
  *           (C) 1998 Monty <xiphmont@mit.edu>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,8 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ *   02110-1301 USA.
  *
  *
  * See ChangeLog for recent changes.
@@ -76,7 +77,7 @@
 #include <fcntl.h>
 #endif
 
-#include <getopt.h>
+#include "getopt.h"
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
@@ -89,6 +90,15 @@
 # include <sys/stat.h>
 #endif
 
+#if !defined(HAVE_GETTIMEOFDAY)
+/* MinGW uses sys/time.h and sys/timeb.h to roll its own gettimeofday() */
+# if defined(HAVE_SYS_TIME_H) && defined(HAVE_SYS_TIMEB_H)
+# include <sys/time.h>
+# include <sys/timeb.h>
+static void gettimeofday(struct timeval* tv, void* timezone);
+# endif
+#endif /* !defined(HAVE_GETTIMEOFDAY) */
+
 #include <cdio/cdio.h>
 #include <cdio/cd_types.h>
 #include <cdio/cdda.h>
@@ -301,6 +311,15 @@ long callbegin;
 long callend;
 long callscript=0;
 
+static int skipped_flag=0;
+static int abort_on_skip=0;
+
+#if TRACE_PARANOIA
+static void
+callback(long int inpos, paranoia_cb_mode_t function)
+{
+}
+#else
 static const char *callback_strings[15]={
   "wrote",
   "finished",
@@ -318,8 +337,6 @@ static const char *callback_strings[15]={
   "duped",
   "transport error"};
 
-static int skipped_flag=0;
-static int abort_on_skip=0;
 static void 
 callback(long int inpos, paranoia_cb_mode_t function)
 {
@@ -331,11 +348,11 @@ callback(long int inpos, paranoia_cb_mode_t function)
 
   int graph=30;
   char buffer[256];
-  static long c_sector=0,v_sector=0;
-  static char dispcache[30]="                              ";
+  static long c_sector=0, v_sector=0;
+  static char dispcache[]="                              ";
   static int last=0;
   static long lasttime=0;
-  long sector,osector=0;
+  long int sector, osector=0;
   struct timeval thistime;
   static char heartbeat=' ';
   int position=0,aheadposition=0;
@@ -543,6 +560,7 @@ callback(long int inpos, paranoia_cb_mode_t function)
   if(function==-1)
     memset(dispcache,' ',graph);
 }
+#endif /* !TRACE_PARANOIA */
 
 const char *optstring = "aBcCd:efg:hi:m:n:o:O:pqQrRsS:Tt:VvwWx:XYZz::";
 
@@ -593,6 +611,17 @@ static char *info_file            = NULL;
   free(p);		 \
   p=NULL;		 
 
+#if !defined(HAVE_GETTIMEOFDAY) && defined(HAVE_SYS_TIME_H) && defined(HAVE_SYS_TIMEB_H)
+static void 
+gettimeofday(struct timeval* tv, void* timezone)
+{ 
+  struct timeb timebuffer;
+  ftime( &timebuffer );
+  tv->tv_sec=timebuffer.time;
+  tv->tv_usec=1000*timebuffer.millitm;
+}
+#endif
+
 /* This is run automatically before leaving the program. 
    Free allocated resources.
 */  
@@ -1063,8 +1092,12 @@ main(int argc,char *argv[])
       paranoia_seek(p,cursor=i_first_lsn,SEEK_SET);      
 
       /* this is probably a good idea in general */
+#if defined(HAVE_GETUID) && defined(HAVE_SETEUID)
       seteuid(getuid());
+#endif
+#if defined(HAVE_GETGID) && defined(HAVE_SETEGID)
       setegid(getgid());
+#endif
 
       /* we'll need to be able to read one sector past user data if we
 	 have a sample offset in order to pick up the last bytes.  We
@@ -1121,16 +1154,16 @@ main(int argc,char *argv[])
 	    if(file[0]=='\0'){
 	      switch (output_type) {
 	      case 0: /* raw */
-		strcat(outfile_name, "cdda.raw");
+		strncat(outfile_name, "cdda.raw", sizeof("cdda.raw"));
 		break;
 	      case 1:
-		strcat(outfile_name, "cdda.wav");
+		strncat(outfile_name, "cdda.wav", sizeof("cdda.wav"));
 		break;
 	      case 2:
-		strcat(outfile_name, "cdda.aifc");
+		strncat(outfile_name, "cdda.aifc", sizeof("cdda.aifc"));
 		break;
 	      case 3:
-		strcat(outfile_name, "cdda.aiff");
+		strncat(outfile_name, "cdda.aiff", sizeof("cdda.aiff"));
 		break;
 	      }
 	    }
@@ -1152,16 +1185,16 @@ main(int argc,char *argv[])
 	  
 	  switch(output_type){
 	  case 0: /* raw */
-	    strcat(outfile_name,"cdda.raw");
+	    strncat(outfile_name, "cdda.raw", sizeof("cdda.raw"));
 	    break;
 	  case 1:
-	    strcat(outfile_name,"cdda.wav");
+	    strncat(outfile_name, "cdda.wav", sizeof("cdda.wav"));
 	    break;
 	  case 2:
-	    strcat(outfile_name,"cdda.aifc");
+	    strncat(outfile_name, "cdda.aifc", sizeof("cdda.aifc"));
 	    break;
 	  case 3:
-	    strcat(outfile_name,"cdda.aiff");
+	    strncat(outfile_name, "cdda.aiff", sizeof("cdda.aiff"));
 	    break;
 	  }
 	  
